****** Cluster-robust inference Stata webinar ********
*** Instructor: Eduardo Garcia ****

clear all
discard


********************************************************************************
************************* Monte Carlo Simulations ******************************
********************************************************************************

program define makedatalinear    /*observations clusters TE*/

	clear
	set obs `1'
			
	generate x1ig = sqrt(3)*rnormal()
	generate x2ig = rchi2(7)
	generate clustervar = runiformint(1, `2')
	scalar threshold = `2' / 3
	generate treatment = (clustervar <= `=scalar(threshold)')
	generate mui = rnormal()
	generate nug = rnormal()
	sort clustervar x1ig
	by clustervar: replace nug = nug[_N]
	
	generate yig = 1 + 1 * x1ig + 1 * x2ig + `3' * treatment + mui + nug	
			
end

program define makedataprobit    /*observations clusters TE*/

	clear
	set obs `1'
			
	generate x1ig = sqrt(3)*rnormal()
	generate x2ig = rchi2(7)
	generate clustervar = runiformint(1, `2')
	scalar threshold = `2' / 3
	generate treatment = (clustervar <= `=scalar(threshold)')
	generate mui = sqrt(1/2)*rnormal()
	generate nug = sqrt(1/2)*rnormal()
	sort clustervar x1ig
	by clustervar: replace nug = nug[_N]
	
	
	generate yig = (1 + 1 * x1ig + 1 * x2ig + `3' * treatment >=  mui + nug)	
			
end


program define doasimlinear      /*observations clusters TE control--crve--hc2*/

	makedatalinear `1' `2' `3'
	
	if `"`4'"' == "control" {
		local control i.clustervar
	}
	if `"`4'"' == "crve" {
		local vce vce(cluster clustervar)
	}
	if `"`4'"' == "hc2" {
		local vce vce(hc2 clustervar, dfadjust)
	}
	
	
	regress yig x1ig x2ig treatment `control', `vce'
	
	scalar beta = r(table)[ "b" , "treatment"]
	scalar ll = r(table)[ "ll" , "treatment"]
	scalar ul = r(table)[ "ul" , "treatment"]
	scalar contained = (ll <= `3')*(ul >= `3')
	
end


program define doasimprobit     /*observations clusters TE  control--crve*/

	makedataprobit `1' `2' `3'
	
	if `"`4'"' == "control" {
		local control i.clustervar
	}
	if `"`4'"' == "crve" {
		local vce vce(cluster clustervar)
	}
	
	probit yig x1ig x2ig treatment `control', `vce'
	
	scalar beta = r(table)[ "b" , "yig: treatment"]
	scalar ll = r(table)[ "ll" , "yig: treatment"]
	scalar ul = r(table)[ "ul" , "yig: treatment"]
	scalar contained = (ll <= `3')*(ul >= `3')
	
end

program define doasimboot   /*observations clusters TE*/

	makedatalinear `1' `2' `3'
	
	wildbootstrap regress yig x1ig x2ig treatment, cluster(clustervar) test(treatment)
	
	scalar beta = r(table)[ "b" , "wb_cns1"]
	scalar ll = r(table)[ "ll" , "wb_cns1"]
	scalar ul = r(table)[ "ul" , "wb_cns1"]
	scalar contained = (ll <= `3')*(ul >= `3')

end


program define montsimlinear      /* observations clusters TE replications control--crve--hc2*/

   capture frame drop mysimlinear

   frame create mysimlinear beta contained
   
   forvalues i = 1(1)`4'  {
   quietly doasimlinear `1' `2' `3' `5'
   frame post mysimlinear (beta) (contained)
   } 
   
   quietly frame mysimlinear : save mysimlinear_results.dta, replace
   quietly use mysimlinear_results.dta, clear 
   sum

end


program define montsimprobit      /* observations clusters TE replications control--crve*/

   capture frame drop mysimprobit

   frame create mysimprobit beta contained
   
   forvalues i = 1(1)`4'  {
   quietly doasimprobit `1' `2' `3' `5'
   frame post mysimprobit (beta) (contained)
   } 
   
   quietly frame mysimprobit : save mysimprobit_results.dta, replace
   quietly use mysimprobit_results.dta, clear 
   sum

end


program define montsimboot      /* observations clusters TE replications */

   capture frame drop mysimboot

   frame create mysimboot beta contained
   
   forvalues i = 1(1)`4'  {
   quietly doasimboot `1' `2' `3'
   frame post mysimboot (beta) (contained)
   } 
   
   quietly frame mysimboot : save mysimboot_results.dta, replace
   quietly use mysimboot_results.dta, clear 
   sum

end

set seed 123456789

*Uncomment to run each simulation

/* 
noisily capture log close
log using cluster_errors, replace

montsimlinear 1000 100 1 1000 
montsimlinear 1000 100 1 1000 control
montsimlinear 1000 100 1 1000 crve

montsimprobit 100000 100 1 1000
montsimprobit 100000 100 1 1000 crve

log close
*/


/*
noisily capture log close
log using cluster_errors_few, replace

montsimlinear 1000 6 1 1000 crve
montsimlinear 1000 6 1 1000 hc2


log close
*/

/*
noisily capture log close
log using cluster_errors_fewboot, replace

montsimboot 1000 21 1 1000

log close
*/


********************************************************************************
**************************** Empirical Examples ********************************
********************************************************************************



noisily capture log close
log using CRVE_conventional, replace

webuse wagework.dta, clear

reg wage tenure market age, vce(cluster personid)
estimates store wage_CRVE

reg wage tenure market age
estimates store wage_iid

etable, estimates(wage_CRVE wage_iid) column(estimates)
log close



noisily capture log close
log using examples_wildboot, replace

webuse wagework.dta, clear

wildbootstrap regress wage tenure, cluster(personid) rseed(12345)
regress wage tenure, vce(cluster personid)

webuse nlsw88.dta, clear

wildbootstrap regress wage tenure, cluster(industry) rseed(12345)
regress wage tenure, vce(cluster industry)

wildbootstrap regress wage tenure age, cluster(industry) rseed(12345)

log close





